// -*- C++ -*-
/*!
 * @file  RTC_Camera_Multi_Viewer.cpp
 * @brief MultiCameraImages入力表示
 * @date $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#include "RTC_Camera_Multi_Viewer.h"

#include <omp.h>
  IplImage* Cam_img[9];
  IplImage* Cam_img_Resize[9];
  //cv::Mat Cam_img[9];
  int Width[9];
  int Height[9];
  int len_img[9];
  int img_FLG[9];
  int OLD_View_FLG=0;
  char winname[90];
  

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_camera_multi_viewer_spec[] =
  {
    "implementation_id", "RTC_Camera_Multi_Viewer",
    "type_name",         "RTC_Camera_Multi_Viewer",
    "description",       "MultiCameraImages入力表示",
    "version",           "1.0.0",
    "vendor",            "AIZUK",
    "category",          "Camera",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.View_FLG", "0",
    "conf.default.WIDTH_HEIGHT_Ratio", "1.0",
    "conf.default.Term_WIDTH_HEIGHT_Ratio", "0.5",
    // Widget
    "conf.__widget__.View_FLG", "text",
    "conf.__widget__.WIDTH_HEIGHT_Ratio", "text",
    "conf.__widget__.Term_WIDTH_HEIGHT_Ratio", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_Camera_Multi_Viewer::RTC_Camera_Multi_Viewer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Camera_Multi_InIn("Camera_Multi_In", m_Camera_Multi_In)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_Camera_Multi_Viewer::~RTC_Camera_Multi_Viewer()
{
}



RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Camera_Multi_In", m_Camera_Multi_InIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("View_FLG", m_View_FLG, "0");
  bindParameter("WIDTH_HEIGHT_Ratio", m_WIDTH_HEIGHT_Ratio, "1.0");
  bindParameter("Term_WIDTH_HEIGHT_Ratio", m_Term_WIDTH_HEIGHT_Ratio, "0.5");
  // </rtc-template>
   m_Camera_Multi_In.images.length(9);
  
  #ifdef _OPENMP
  std::cout<<"The number of processors is "<<omp_get_num_procs()<<std::endl;
  #endif
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onFinalize()
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onExecute(RTC::UniqueId ec_id)
{
  int i=0;
  int Term_Width=0;
  int Term_Height=0;
IplImage *tmp[4];
CvRect roi = cvRect(0, 0, 0, 0);
  if(m_Camera_Multi_InIn.isNew()){
  	m_Camera_Multi_InIn.read();
  	for(i=0;i<9;i++){
  		img_FLG[i]=0;
		len_img[i] =m_Camera_Multi_In.images[i].pixels.length();
  		if(!len_img[i] > 0){
  			//std::cout<<"ERRRR"<<std::endl;
  		}else{
	  	//	Cam_img[i].width = m_Camera_Multi_In.images[i].width ;
		//	Cam_img[i].height = m_Camera_Multi_In.images[i].height;
		//	
		    if(Height[i]!= (int)m_Camera_Multi_In.images[i].height
		    ||
		     Width[i] != (int)m_Camera_Multi_In.images[i].width)
		    {
			std::cout<<"WH:"
			<<i
			<<"DD"
			<<m_Camera_Multi_In.images[i].width
			<<":"
			<<m_Camera_Multi_In.images[i].height
			<<std::endl;
			Width[i]=m_Camera_Multi_In.images[i].width;
			Height[i]=m_Camera_Multi_In.images[i].height;
			
			
			Cam_img[i]= cvCreateImage(
								cvSize(
								Width[i],
								Height[i]), 
								IPL_DEPTH_8U, 
								3);
			}
			
			//std::cout<<"HHHhhhhhhhhhhhhhhhhhh"<<std::endl;
			memcpy(Cam_img[i] -> imageData
			, (void *)&(m_Camera_Multi_In.images[i].pixels[0])
			,  m_Camera_Multi_In.images[i].pixels.length());
	  		img_FLG[i]=1;
			Cam_img_Resize[i] = cvCreateImage(
										cvSize(
										Width[i]*m_Term_WIDTH_HEIGHT_Ratio ,
										Height[i]*m_Term_WIDTH_HEIGHT_Ratio), 
										IPL_DEPTH_8U, 
										3);
	
			cvResize(Cam_img[i], Cam_img_Resize[i], CV_INTER_LINEAR);
		}
  		
	}
		
/*		
va_list list;
IplImage *tmp[4];
IplImage *Term_img;
CvRect roi = cvRect(0, 0, 0, 0);
*/
	if(m_View_FLG==0){
		if(OLD_View_FLG==1){
			std::cout<<"WH:"<<std::endl;
			sprintf(winname,"Term");
	  		cvDestroyWindow(winname);	
		}
		for(i=0;i<9;i++){
		  	//Term_Width +=Cam_img_Resize[i]->width;
		  	//Term_Height =Cam_img_Resize[i]->height;
			if(img_FLG[i] == 1){
				sprintf(winname,"Image[%d]",i );
				cvShowImage(winname,Cam_img[i]);
	
			}
		}
		OLD_View_FLG=m_View_FLG;
	}else if(m_View_FLG==1){

			if(OLD_View_FLG==0){
			  	for(i=0;i<9;i++){
			  		if(img_FLG[i] == 1){
					std::cout<<"MON"<<std::endl;
						sprintf(winname,"Image[%d]",i );
						cvDestroyWindow(winname);
					}
			  	}
			}
			Term_Width   =Cam_img_Resize[0]->width*2;
			Term_Height  =Cam_img_Resize[0]->height*3;

			IplImage *Term_img;
			Term_img = cvCreateImage (cvSize (Term_Width, Term_Height), IPL_DEPTH_8U, 3);
			cvZero (Term_img);

			  for (i = 0; i < 4; i++) {
			    roi.width = Cam_img_Resize[i]->width;
			    roi.height = Cam_img_Resize[i]->height;
			    if(i==0){
			    	roi.x = (int)(roi.width/2);
			    	roi.y =0;// roi.height;
			    }
			    if(i==1){
			    	roi.x = 0;
			    	roi.y = roi.height;
			    }
			    if(i==2){
			    	roi.x = roi.width;
			    	roi.y = roi.height;
			    }
			    if(i==3){
			    	roi.x = (int)(roi.width/2);
			    	roi.y = roi.height*2;
			    } 
			    cvSetImageROI (Term_img, roi);
			    cvCopy (Cam_img_Resize[i], Term_img);

			  cvResetImageROI (Term_img);
			  }

		
			sprintf(winname,"Term");
			cvShowImage(winname,Term_img);	
			cvReleaseImage(&Term_img);
		//	for(i=0;i<9;i++){
		//		if(img_FLG[i] == 1)cvReleaseImage(&Cam_img[i]);
		//	}
			}
		OLD_View_FLG=m_View_FLG;
	}
	
	int c = cv::waitKey(2);
  	for(i=0;i<9;i++){
		if(img_FLG[i] == 1)cvReleaseImage(&Cam_img_Resize[i]);
	}

  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi_Viewer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



extern "C"
{
 
  void RTC_Camera_Multi_ViewerInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_camera_multi_viewer_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_Camera_Multi_Viewer>,
                             RTC::Delete<RTC_Camera_Multi_Viewer>);
  }
  
};


